#======================================================================
#
#  PROGRAM NAME:  RasPyiet.py
#
#======================================================================
#  DESCRIPTION: Graphical User Interface for controlling
#               ROMIO from a Raspberry Pi Zero 2 W
#======================================================================
#  REQU. LIBRARIES: Tinker, OpenCV, gpiozero
#
#======================================================================
# Author:        DaniloR29
# Version:       1.0
# Copyright (C): 2022 Danilo29R
# Distributed under terms of the MIT license.
#======================================================================


from gpiozero import Servo, DistanceSensor
from gpiozero.pins.pigpio import PiGPIOFactory

from time import sleep
import cv2

import tkinter as tk
from tkinter import *
from tkinter import ttk

from PIL import Image, ImageTk

#
# Servos setup
#

factory = PiGPIOFactory()
servo = Servo(17,  pin_factory=factory)
servo1 = Servo(18,  pin_factory=factory)
servo2 = Servo(23,  pin_factory=factory)

# initialize the camera

vd=0
Speed=0.25
FStepX=25
posx=0
cam = cv2.VideoCapture(0)   # USB microscope
cam1 = cv2.VideoCapture(2)  # Raspberry Camera

def ServoPositionY(posy):
    vv=float(posy) / 90.
    servo.value = vv
    sleep(0.1)
    picture=TakePicture()
    label.configure(image=picture)
    label.image= picture
    s="Y steps: "+str(int(100*(vv)))
    tposy.configure(text=s)

def ServoPosition3(posf):
    servo2.value = float(posf) / 90
    sleep(0.1)
    picture=TakePicture()
    label.configure(image=picture)
    label.image= picture
    s="Focus: "+str(int(100*(float(posf) / 90)))
    Focus.configure(text=s)

def MoveL():
    global posx
    servo1.value = -Speed
    for i in range(0,FStepX):
        servo1.value = -Speed
    servo1.value = 0
    sleep(0.1)
    picture= TakePicture()
    label.configure(image=picture)
    label.image= picture
    posx=posx-1
    s="X steps: "+str(posx)
    tposx.configure(text=s)

def MoveR():
    global posx
    servo1.value = Speed
    for i in range(0,FStepX):
        servo1.value = Speed
    servo1.value = 0
    sleep(0.1)
    picture= TakePicture()
    label.configure(image=picture)
    label.image= picture
    posx=posx+1
    s="X steps: "+str(posx)
    tposx.configure(text=s)

def Refresh():
    picture=TakePicture()
    label.configure(image=picture)
    label.image= picture

def SetBound(sel):
    global posx
    if (sel == 0):
        exmin.delete(0,"end")
        exmin.insert(0,str(posx))
    if (sel == 1):
        exmax.delete(0,"end")
        exmax.insert(0,str(posx))
    if (sel == 2):
        eymin.delete(0,"end")
        eymin.insert(0,str(int(100*float(posy.get())/90)))
    if (sel == 3):
        eymax.delete(0,"end")
        eymax.insert(0,str(int(100*float(posy.get())/90)))

def TakePicture():
    ret, src = cam.read()
    if (vd):
        ret, src = cam1.read()
    image = cv2.rotate(src, cv2.cv2.ROTATE_180)
    # display the image on screen and wait for a keypress
    dim = (480, 360)
    # resize image
    rimg = cv2.resize(image, dim, interpolation = cv2.INTER_AREA)
    b,g,r = cv2.split(rimg)
    img2 = cv2.merge((r,g,b))
    img3=Image.fromarray(img2)
    return ImageTk.PhotoImage(img3)

def Starting():
    global xp
    xp=posx
    if (xp > xmi):
        sp=-Speed
        for i in range(xp,xmi,-1):
            MoveX(sp)
    if (xp <= xmi):
        sp=Speed
        for i in range(xp,xmi,1):
            MoveX(sp)

def MoveX(sp):
    servo1.value = sp
    for i in range(0,FStepX):
        servo1.value = sp
    servo1.value = 0
    sleep(0.1)
    picture= TakePicture()
    label.configure(image=picture)
    label.image= picture

def SetZeroX():
    global posx
    posx=0
    s="X steps: "+str(posx)
    tposx.configure(text=s)

def ScanningN():
    # Scanning forg sg90 servo
    global xmi,xma
    xmi=int(float(xmin.get()))
    xma=int(float(xmax.get()))
    ymi=int(float(ymin.get()))
    yma=int(float(ymax.get()))
    xi=int(float(xinc.get()))
    yi=int(float(yinc.get()))
    print (xmi," ",xma,"  ",xi)
    # Move the X axis to the minimum value
    Starting()
    ii=0
    jj=0
    sp=Speed
    cc=-1
    for i in range(ymi,yma,yi):
        jj=0
        vv =i*1.0
        posy=vv
        print (float(posy))
        ServoPositionY(posy)
        sleep(2.0)
        if (cc==-1):
            cc=1
        elif (cc==1):
            cc=-1
        if (cc==-1):
            nn=0
            for j in range(xmi,xma,1):
                sp=Speed
                nn=nn+1
                if (nn%xi==0):
                    ScanStepX(sp,ii,jj)
                    jj=jj+1
        elif (cc==1):
            nn=0
            for j in range(xma,xmi,-1):
                sp=-Speed
                nn=nn+1
                if (nn%xi==0):
                    ScanStepX(sp,ii,jj)
                    jj=jj+1
        ii=ii+1

def ScanStepX(sp,ii,jj):
        sleep(1.0)
        ret, image = cam.read()
        filen=fname.get()+str(ii)+"-"+str(jj)+".png"
        print (ii," ",jj,"  ",filen)
        MoveX(sp)
        root.update()
        cv2.imwrite(filen,image)

def SavePic():
    ret, image = cam.read()
    filen=fname.get()+".png"
    cv2.imwrite(filen,image)

def Quit():
    exit()

def SwapVideo():
    global vd
    if (vd==0):
        vd=1
    elif (vd==1):
        vd=0
    Refresh()

# Start the Tinker GUI

# root window
root = tk.Tk()
root.geometry("500x650")
root.title('ROMIO: The REmotely COntrolled MIcroscOpe')
root.resizable(0, 0)
root.configure(bg="cyan")
posy = DoubleVar()
posf = DoubleVar()

xmin=tk.StringVar()
xmax=tk.StringVar()
ymin=tk.StringVar()
ymax=tk.StringVar()
xinc=tk.StringVar()
yinc=tk.StringVar()
fname=tk.StringVar()

# Setting initial values
xinc.set("1")
yinc.set("20")
fname.set("mimage")

xmin.set("-5")
xmax.set("5")
ymin.set("-65")
ymax.set("65")

# configure the grid
root.columnconfigure(0, weight=1)
root.columnconfigure(1, weight=3)

picture=TakePicture()
label = ttk.Label(root,image=picture)
label.grid(row=1)

label.image=picture
tposx = tk.Label(root, text= "X steps: 0",bg="cyan")
tposx.grid(row=2,column=0,sticky='NSW')

tposy = Label(root, text="Y steps: 0",bg="cyan")
tposy.grid(row=2,column=0,sticky='NSE')

servo_positionY  = ttk.Scale(root, variable=posy,from_=-80, to=80, command=ServoPositionY, orient=VERTICAL)
servo_positionY.grid(row=1,column=1,sticky='NSW')

Focus = tk.Label(root, text="Focus: 0",bg="cyan")
Focus.grid(row=3,column=0,sticky='NSW')

servo_position3= ttk.Scale(root, variable=posf,length=400,from_=-90, to=90, command=ServoPosition3)
servo_position3.grid(row=3,column=0,sticky='NSE')

l5=Label(root,text="X increment:",bg="cyan").grid(row=8,column=0,sticky="W")
l6=Label(root,text="Y increment:",bg="cyan").grid(row=9,column=0,sticky="W")
exmin = ttk.Entry(root,textvariable=xmin)
exmin.grid(row=4,column=0)
exmax = ttk.Entry(root,textvariable=xmax)
exmax.grid(row=5,column=0)
eymin = ttk.Entry(root,textvariable=ymin)
eymin.grid(row=6,column=0)
eymax = ttk.Entry(root,textvariable=ymax)
eymax.grid(row=7,column=0)
exinc = ttk.Entry(root,textvariable=xinc)
exinc.grid(row=8,column=0)
eyinc = ttk.Entry(root,textvariable=yinc)
eyinc.grid(row=9,column=0,)

b1=Button(root,bg="yellow",width=15,text="Set Xmin",command=lambda sel=0 : SetBound(sel))
b1.grid(row=4,column=0,sticky="W")
b2=Button(root,bg="yellow",width=15,text="Set Xmax",command=lambda sel=1 : SetBound(sel))
b2.grid(row=5,column=0,sticky="W")
b3=Button(root,bg="yellow",width=15,text="Set Ymin",command=lambda sel=2 : SetBound(sel))
b3.grid(row=6,column=0,sticky="W")
b4=Button(root,bg="yellow",width=15,text="Set Ymax",command=lambda sel=3 : SetBound(sel))
b4.grid(row=7,column=0,sticky="W")

lfname = Label(root,text="                   File name: ",bg="cyan")
lfname.grid(row=10,sticky="W")

ifname  = ttk.Entry(root,textvariable=fname)
ifname.grid(row=10)

button1 = Button(root,command=Refresh,text="Refresh",width=15,bg="magenta")
button1.grid(row=8,sticky="E")
button2 = Button(root,command=SavePic,text="Save a photo",width=15,bg="pink")
button2.grid(row=9,sticky="E")
button3 = Button(root,command=ScanningN,text="Scanning",width=15,bg="yellow")
button3.grid(row=7,sticky="E")
button4 = Button(root,command=Quit,text="Quit",bg="red")
button4.grid(row=10,sticky="W",pady=4)

button5 = Button(root,command=MoveL,text="Move Left",width=15,bg="gold")
button5.grid(row=4,sticky="E",pady=4)
button6 = Button(root,command=MoveR,text="Move Right",width=15,bg="gold")
button6.grid(row=5,sticky="E",pady=4)
button7 = Button(root,command=SetZeroX,text="Set Zero X",width=15,bg="gold")
button7.grid(row=6,sticky="E",pady=4)

button8 = Button(root,command=SwapVideo,text="SwapVideo",bg="green")
button8.grid(row=10,sticky="E",pady=4)

root.mainloop()

